
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2021-09-10T07:14:13-0400",
 "dirty": false,
 "error": null,
 "full-revisionid": "c12e1d6f1c13f5c94a0bbe953e9a7f1858f45c5e",
 "version": "0.1.0a32"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
